@@ -22,6 +22,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <glib/gstdio.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
@@ -36,10 +37,13 @@
 #include "ev-init.h"
 #include "ev-file-helpers.h"
 #include "ev-stock-icons.h"
+
+#ifdef WITH_SMCLIENT
 #include "eggsmclient.h"
-#ifndef G_OS_WIN32
+#ifdef GDK_WINDOWING_X11
 #include "eggdesktopfile.h"
 #endif
+#endif /* WITH_SMCLIENT */
 
 static gchar   *ev_page_label;
 static gchar   *ev_find_string;
@@ -76,6 +80,62 @@ static const GOptionEntry goption_options[] =
 	{ NULL }
 };
 
+static gboolean
+launch_previewer (void)
+{
+	GString *cmd_str;
+	gchar   *cmd;
+	gint     argc;
+	gchar  **argv;
+	gboolean retval = FALSE;
+	GError  *error = NULL;
+
+	/* Rebuild the command line, ignoring options
+	 * not supported by the previewer and taking only
+	 * the first path given
+	 */
+	cmd_str = g_string_new ("evince-previewer");
+		
+	if (print_settings) {
+		gchar *quoted;
+
+		quoted = g_shell_quote (print_settings);
+		g_string_append_printf (cmd_str, " --print-settings %s", quoted);
+		g_free (quoted);
+	}
+
+	if (unlink_temp_file)
+		g_string_append (cmd_str, " --unlink-tempfile");
+
+	if (file_arguments) {
+		gchar *quoted;
+		
+		quoted = g_shell_quote (file_arguments[0]);
+		g_string_append_printf (cmd_str, " %s", quoted);
+		g_free (quoted);
+	}
+
+	cmd = g_string_free (cmd_str, FALSE);
+	g_shell_parse_argv (cmd, &argc, &argv, &error);
+	g_free (cmd);
+	
+	if (!error) {
+		retval = gdk_spawn_on_screen (gdk_screen_get_default (),
+					      NULL, argv, NULL,
+					      G_SPAWN_SEARCH_PATH,
+					      NULL, NULL, NULL,
+					      &error);
+		g_strfreev (argv);
+	}
+
+	if (error) {
+		g_warning ("Error launching previewer: %s\n", error->message);
+		g_error_free (error);
+	}
+
+	return retval;
+}
+
 static void
 value_free (GValue *value)
 {
@@ -96,8 +156,6 @@ value_free (GValue *value)
  *                          the page of the document to display.
  *  mode                ->  only if the view mode is one of the availables,
  *                          the view mode.
- *  unlink-temp-file    ->  only if the view mode is preview mode and
- *                          unlink-temp-file has been passed, unlink-temp-file.
  *
  * Returns: a pointer into #GHashTable with data from the arguments.
  */
@@ -159,8 +217,6 @@ arguments_parse (void)
 		mode = EV_WINDOW_MODE_FULLSCREEN;
 	else if (presentation_mode)
 		mode = EV_WINDOW_MODE_PRESENTATION;
-	else if (preview_mode)
-		mode = EV_WINDOW_MODE_PREVIEW;
 	else
 		return args;
 
@@ -170,39 +226,30 @@ arguments_parse (void)
 
 	g_hash_table_insert (args, g_strdup ("mode"), value);
 
-	if (mode == EV_WINDOW_MODE_PREVIEW && unlink_temp_file) {
-		value = g_new0 (GValue, 1);
-		g_value_init (value, G_TYPE_BOOLEAN);
-		g_value_set_boolean (value, unlink_temp_file);
-
-		g_hash_table_insert (args,
-				     g_strdup ("unlink-temp-file"),
-				     value);
-	}
-
-	if (mode == EV_WINDOW_MODE_PREVIEW && print_settings) {
-		value = g_new0 (GValue, 1);
-		g_value_init (value, G_TYPE_STRING);
-		g_value_set_string (value, print_settings);
-
-		g_hash_table_insert (args,
-				     g_strdup ("print-settings"),
-				     value);
-		g_free (print_settings);
-		print_settings = NULL;
-	}
-
 	return args;
 }
 
+static gint
+find_window_list (EvWindow    *window,
+		  const gchar *uri)
+{
+	return g_ascii_strcasecmp (uri, ev_window_get_uri (window));
+}
+
 static void
 load_files (const char **files,
 	    GHashTable  *args)
 {
-	int i;
+	int    i;
+	GList *windows;
+
+	windows = ev_application_get_windows (EV_APP);
 
 	if (!files) {
-		ev_application_open_window (EV_APP, args, GDK_CURRENT_TIME, NULL);
+		if (!windows)
+			ev_application_open_window (EV_APP, args, GDK_CURRENT_TIME, NULL);
+		else
+			g_list_free (windows);
 		return;
 	}
 
@@ -215,7 +262,12 @@ load_files (const char **files,
 		file = g_file_new_for_commandline_arg (files[i]);
 		uri = g_file_get_uri (file);
 		g_object_unref (file);
-		
+
+		if (g_list_find_custom (windows, uri, (GCompareFunc) find_window_list)) {
+			g_free (uri);
+			continue;
+		}
+
 		label = strchr (uri, '#');
 
 		if (label) {
@@ -241,6 +293,8 @@ load_files (const char **files,
 		
 		g_free (uri);
         }
+
+	g_list_free (windows);
 }
 
 #ifdef ENABLE_DBUS
@@ -347,8 +401,11 @@ main (int argc, char *argv[])
 	context = g_option_context_new (N_("GNOME Document Viewer"));
 	g_option_context_set_translation_domain(context, GETTEXT_PACKAGE);
 	g_option_context_add_main_entries (context, goption_options, GETTEXT_PACKAGE);
-	
+
+#ifdef WITH_SMCLIENT
 	g_option_context_add_group (context, egg_sm_client_get_option_group ());
+#endif
+
 	g_option_context_add_group (context, gtk_get_option_group (TRUE));
 
 	if (!g_option_context_parse (context, &argc, &argv, &error)) {
@@ -360,6 +417,14 @@ main (int argc, char *argv[])
 	}
 	g_option_context_free (context);
 
+	if (preview_mode) {
+		gboolean retval;
+		
+		retval = launch_previewer ();
+		
+		return retval ? 0 : 1;
+	}
+
 	args = arguments_parse ();
 
 #ifdef ENABLE_DBUS
@@ -371,27 +436,33 @@ main (int argc, char *argv[])
 		}
 	}
 #endif /* ENABLE_DBUS */
-
+	
         if (!ev_init ())
                 return 1;
 
 	ev_stock_icons_init ();
 
-#ifdef G_OS_WIN32
-	/* Manually set name and icon in win32 */
-	g_set_application_name (_("Evince"));
-	gtk_window_set_default_icon_name ("evince");
-#else
+#if defined(WITH_SMCLIENT) && defined(GDK_WINDOWING_X11)
 	egg_set_desktop_file (GNOMEDATADIR "/applications/evince.desktop");
-#endif /* G_OS_WIN32 */
+#else
+	/* Manually set name and icon */
+	g_set_application_name (_("Document Viewer"));
+	gtk_window_set_default_icon_name ("evince");
+#endif /* WITH_SMCLIENT && GDK_WINDOWING_X11 */
 
-	if (!ev_application_load_session (EV_APP))
-		load_files (file_arguments, args);
+	ev_application_load_session (EV_APP);
+	load_files (file_arguments, args);
 	g_hash_table_destroy (args);
 
+	/* Change directory so we don't prevent unmounting in case the initial cwd
+	 * is on an external device (see bug #575436)
+	 */
+	g_chdir (g_get_home_dir ());	
+
 	gtk_main ();
 
 	ev_shutdown ();
+	ev_stock_icons_shutdown ();
 
 	return 0;
 }
